#include "common.h"

struct Appdata
{
    float3 Position	    : POSITION;
    float3 Normal	    : NORMAL;
    float2 Uv	        : TEXCOORD0;
    int4   ColorExtra	: COLOR0;
};

struct VertexOutput
{
    float4 HPosition    : POSITION;
    float2 Uv           : TEXCOORD0;
    float4 Diffuse      : COLOR0;
    float4 Specular     : COLOR1;

#ifdef PIN_GBUFFER
    float Depth         : TEXCOORD1;
#endif
};

uniform float3 CameraPosition;
uniform float4x4 ViewProjection;

uniform float3 Lamp0Dir;
uniform float3 Lamp1Dir;
uniform float3 Lamp0Color;
uniform float3 Lamp1Color;
uniform float3 AmbientColor;

uniform float4x4 WorldMatrixArray[32];

VertexOutput DefaultVS(Appdata IN)
{
    VertexOutput OUT = (VertexOutput)0;

    float4x4 worldMatrix = WorldMatrixArray[IN.ColorExtra.a];
		
	float3 posWorld = mul(worldMatrix, float4(IN.Position.xyz, 1)).xyz;
    float3 normalWorld = mul(worldMatrix, float4(IN.Normal, 0)).xyz;

    float3 view = normalize(CameraPosition - posWorld.xyz);

    float specularPower = 50.0;

    float4 lit0 = lit(dot(normalWorld, -Lamp0Dir), dot(normalize(-Lamp0Dir + view), normalWorld), specularPower);

    float3 ambient = AmbientColor;
	float3 diffuse = lit0.y * Lamp0Color + max(dot(normalWorld, -Lamp1Dir), 0) * Lamp1Color;
    float3 specular = lit0.z * Lamp0Color;

	OUT.HPosition = mul(ViewProjection, float4(posWorld, 1));
	OUT.Uv = IN.Uv;

    float3 color = IN.ColorExtra.rgb / 255.f;

    OUT.Diffuse = float4((ambient + diffuse) * color, 1);
    OUT.Specular = float4(specular * color, 0);

#ifdef PIN_GBUFFER
	OUT.Depth = OUT.HPosition.w;
#endif

	return OUT;
}

sampler2D DiffuseMap: register(s0);

void DefaultPS(VertexOutput IN,
#ifdef PIN_GBUFFER
    out float4 oColor1: COLOR1,
#endif
    out float4 oColor0: COLOR0)
{
    float4 diffuse = tex2D(DiffuseMap, IN.Uv) * IN.Diffuse;
    float4 specular = IN.Specular;

    oColor0 = diffuse + specular;

#ifdef PIN_GBUFFER
    oColor1 = gbufferPack(IN.Depth, diffuse.rgb, specular.rgb);
#endif
}
